<?php

namespace Axepta\Payment;

use Exception;

/**
 * Class Api
 *
 * @package Axepta\Payment
 */
class Api
{

	const TRANSACTION = 'transaction';
	const PAYMENT_LINK = 'payment-link';
	const PROFILE = 'profile';
	const MERCHANT = 'merchant';
	const SERVICE = 'service';
	const TRANSACTION_TYPE_SALE = 'sale';
	const TRANSACTION_TYPE_REFUND = 'refund';

	/**
	 * @var array
	 */
	private static $serviceUrls = [
		Util::ENVIRONMENT_PRODUCTION => 'https://api.axepta.pl/v1',
		Util::ENVIRONMENT_SANDBOX    => 'https://api.sandbox.axepta.pl/v1',
	];

	/**
	 * @var string
	 */
	private $authorizationToken;

	/**
	 * @var string
	 */
	private $merchantId;

	/**
	 * @var string
	 */
	private $serviceId;

	/**
	 * @var string
	 */
	private $environment;

	/**
	 * Constructor
	 *
	 * @param string $authorizationToken
	 * @param string $merchantId
	 * @param string $serviceId
	 * @param string $environment
	 */
	public function __construct(
		$authorizationToken,
		$merchantId,
		$serviceId,
		$environment = ''
	) {

		$this->authorizationToken = $authorizationToken;
		$this->merchantId = $merchantId;
		$this->serviceId = $serviceId;

		$this->environment = $environment
			?: Util::ENVIRONMENT_PRODUCTION;
	}

	/**
	 * @param string $id
	 *
	 * @return array
	 */
	public function getTransaction($id)
	{

		return $this->call(
			$this->getTransactionUrl($id),
			Util::METHOD_REQUEST_GET
		);
	}

	/**
	 * @param string $url
	 * @param string $methodRequest
	 * @param string $body
	 *
	 * @return array
	 */
	private function call($url, $methodRequest, $body = '')
	{

		$curl = curl_init($url);
		curl_setopt($curl, CURLOPT_CUSTOMREQUEST, $methodRequest);
		curl_setopt($curl, CURLOPT_POSTFIELDS, $body);
		curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
		curl_setopt($curl, CURLOPT_CONNECTTIMEOUT, 10);
		curl_setopt($curl, CURLOPT_TIMEOUT, 10);
		curl_setopt($curl, CURLOPT_HTTPHEADER, [
			'Content-Type: application/json',
			'Authorization: Bearer ' . $this->authorizationToken,
		]);

		$resultCurl = json_decode(curl_exec($curl), true);

		$httpCode = curl_getinfo($curl, CURLINFO_HTTP_CODE);

		if(($httpCode !== 200) || !$resultCurl) {

			return [
				'success' => false,
				'data'    => [
					'httpCode'  => $httpCode,
					'curlError' => curl_error($curl),
					'body'      => $resultCurl,
				],
			];
		}

		return [
			'success' => true,
			'body'    => $resultCurl,
		];
	}

	/**
	 * @return string
	 */
	private function getTransactionUrl($id)
	{

		$baseUrl = self::getServiceUrl();

		if($baseUrl) {
			return $baseUrl
				. '/'
				. self::MERCHANT
				. '/'
				. $this->merchantId
				. '/'
				. self::TRANSACTION
				. '/'
				. $id;
		}

		return '';
	}

	/**
	 * @return string
	 */
	private function getServiceUrl()
	{

		if(isset(self::$serviceUrls[$this->environment])) {
			return self::$serviceUrls[$this->environment];
		}

		return '';
	}

	/**
	 * @param int    $amount
	 * @param string $currency
	 * @param string $orderId
	 * @param string $returnUrl
	 * @param string $successReturnUrl
	 * @param string $failureReturnUrl
	 * @param string $customerFirstName
	 * @param string $customerLastName
	 * @param string $customerEmail
	 * @param string $phone
	 * @param string $visibleMethod
	 * @param array  $cart
	 * @param array  $invoice
	 *
	 * @return string
	 */
	public function prepareDataPaymentLink(
		$amount,
		$currency,
		$orderId,
		$returnUrl,
		$successReturnUrl,
		$failureReturnUrl,
		$customerFirstName,
		$customerLastName,
		$customerEmail,
		$notificationUrl = '',
		$phone = '',
		$creditType = [],
		$visibleMethod = [],
		$cart = [],
		$invoice = [],
		$preselectMethodCode = ''
	) {

		$array = [
			'serviceId'        => $this->serviceId,
			'amount'           => Util::convertAmountToFractional($amount),
			'currency'         => $currency,
			'orderId'          => (string) $orderId,
			'title'            => (string) $orderId,
			'successReturnUrl' => $successReturnUrl,
			'failureReturnUrl' => $failureReturnUrl,
			'customer'         => [
				'firstName' => $customerFirstName,
				'lastName'  => $customerLastName,
				'email'     => $customerEmail,
			],
		];

		if($returnUrl) {
			$array['returnUrl'] = $returnUrl;
		}

		if($notificationUrl) {
			$array['notificationUrl'] = $notificationUrl;
		}

		if($phone) {
			$array['customer']['phone'] = $phone;
		}

		if($visibleMethod) {
			$array['visibleMethod'] = $visibleMethod;
		}

		if($cart) {
			$array['cart'] = $cart;
		}

		if($invoice) {
			$array['invoice'] = $invoice;
		}

		if($preselectMethodCode) {
			$array['preselectMethodCode'] = $preselectMethodCode;
		}

		if($creditType) {
			$array['creditTypes'] = $creditType;
		}

		return json_encode($array);
	}

	/**
	 * @param string $body
	 *
	 * @return array
	 */
	public function createTransaction($body, $url = '', $method = '')
	{
		if(!$url) {
			$url = $this->getTransactionCreateUrl();
		}
		if(!$method) {
			$method = Util::METHOD_REQUEST_POST;
		}

		return $this->call(
			$url,
			$method,
			$body
		);
	}

	/**
	 * @param string $body
	 *
	 * @return array
	 */
	public function createPaymentLink($body, $url = '', $method = '')
	{
		if(!$url) {
			$url = $this->getPaymentLinkCreateUrl();
		}

		if(!$method) {
			$method = Util::METHOD_REQUEST_POST;
		}

		return $this->call(
			$url,
			$method,
			$body
		);
	}

	/**
	 * @return string
	 */
	private function getTransactionCreateUrl()
	{

		$baseUrl = self::getServiceUrl();

		if($baseUrl) {
			return $baseUrl
				. '/'
				. self::MERCHANT
				. '/'
				. $this->merchantId
				. '/'
				. self::TRANSACTION;
		}

		return '';
	}

	/**
	 * @return string
	 */
	private function getPaymentLinkCreateUrl()
	{

		$baseUrl = self::getServiceUrl();

		if($baseUrl) {
			return $baseUrl
				. '/'
				. self::MERCHANT
				. '/'
				. $this->merchantId
				. '/'
				. self::PAYMENT_LINK;
		}

		return '';
	}

	/**
	 * @return array
	 */
	public function getServiceInfo()
	{
		return $this->call(
			$this->getServiceInfoUrl(),
			Util::METHOD_REQUEST_GET
		);
	}

	/**
	 * @return string
	 */
	private function getServiceInfoUrl()
	{

		$baseUrl = self::getServiceUrl();

		if($baseUrl) {
			return $baseUrl
				. '/'
				. self::MERCHANT
				. '/'
				. $this->merchantId
				. '/'
				. self::SERVICE
				. '/'
				. $this->serviceId;
		}

		return '';
	}

	/**
	 * @param string $body
	 *
	 * @return array
	 */
	public function createRefund($body, $transactionUuid)
	{

		return $this->call(
			$this->getRefundCreateUrl($transactionUuid),
			Util::METHOD_REQUEST_POST,
			$body
		);
	}

	/**
	 * @param string $transactionUuid
	 *
	 * @return string
	 */
	private function getRefundCreateUrl($transactionUuid)
	{

		$baseUrl = $this->getServiceUrl();

		if($baseUrl) {
			return $baseUrl
				. '/'
				. self::MERCHANT
				. '/'
				. $this->merchantId
				. '/'
				. self::TRANSACTION
				. '/'
				. $transactionUuid
				. '/refund';
		}

		return '';
	}

	/**
	 * @param array $transaction
	 *
	 * @return array
	 */
	public static function parseStringToArray($transaction)
	{

		$array = [];

		if($transaction['action']['method'] === Util::METHOD_REQUEST_POST) {
			parse_str($transaction['action']['contentBodyRaw'], $array);
		}

		if($transaction['action']['method'] === Util::METHOD_REQUEST_GET) {
			$urlParsed = parse_url($transaction['action']['url']);

			if(isset($urlParsed['query']) && $urlParsed['query']) {
				parse_str($urlParsed['query'], $array);

				return $array;
			}

			if(isset($transaction['action']['contentBodyRaw']) && $transaction['action']['contentBodyRaw']) {
				parse_str($transaction['action']['contentBodyRaw'], $array);
			}
		}

		return $array;
	}

	/**
	 * @param array  $transactionLimits
	 * @param number $total
	 *
	 * @return bool
	 */
	public function verifyTransactionLimits($transactionLimits, $total)
	{

		$isMaxValue = isset($transactionLimits['maxTransaction']['value']) && $transactionLimits['maxTransaction']['value'];
		$isMinValue = isset($transactionLimits['minTransaction']['value']) && $transactionLimits['minTransaction']['value'];

		if($isMaxValue || $isMinValue) {
			$total = Util::convertAmountToFractional($total);

			// leave this condition - in 99% of cases, it is going to be true, so there's no need to check another scenario
			if($isMaxValue && $isMinValue) {
				return $total >= $transactionLimits['minTransaction']['value'] && $total <= $transactionLimits['maxTransaction']['value'];
			}

			if($isMaxValue) {
				return $total <= $transactionLimits['maxTransaction']['value'];
			}

			if($isMinValue) {
				return $total >= $transactionLimits['minTransaction']['value'];
			}
		}

		return false;
	}
}
